@@ -29,6 +29,7 @@
 #include <unistd.h>
 #include <libintl.h>
 #include <locale.h>
+#include <linux/limits.h>
 
 #include <blkid/blkid.h>
 
@@ -316,7 +317,7 @@ static stringList_t _zuluCryptVolumeList_0( int resolve_loop_devices )
 
 					e = zuluCryptLoopDeviceAddress_1( device ) ;
 
-					if( StringListHasNoEntry( stz,e ) ){
+					if( !zuluCryptBitLockerVolume( e ) && StringListHasNoEntry( stz,e ) ){
 
 						/*
 						 * Here we only keep one loop device if the volume file has
@@ -822,12 +823,9 @@ void zuluCryptPrintPartitionProperties( const char * device )
 		}else{
 			printf( "Nil\t" ) ;
 		}
-		if( blkid_probe_lookup_value( blkid,"TYPE",&e,NULL ) == 0 ){
 
-			printf( "%s\t",e ) ;
-		}else{
-			printf( "Nil\t" ) ;
-		}
+		printf( "%s\t",zuluCryptVolumeType( blkid,device ) ) ;
+
 		if( blkid_probe_lookup_value( blkid,"UUID",&e,NULL ) == 0 ){
 
 			printf( "%s\n",e ) ;
