@@ -43,7 +43,7 @@
 #include "ui_password.h"
 #include "openvolume.h"
 #include "tcrypt.h"
-#include "task.h"
+#include "task.hpp"
 #include "utility.h"
 #include "dialogmsg.h"
 #include "plugin_path.h"
@@ -72,6 +72,8 @@ passwordDialog::passwordDialog( QTableWidget * table,
 {
 	m_ui->setupUi( this ) ;
 
+	m_ui->PassPhraseField->setMaxLength( 32767 ) ;
+
 	m_parent = parent ;
 
 	this->setFixedSize( this->size() ) ;
@@ -79,6 +81,8 @@ passwordDialog::passwordDialog( QTableWidget * table,
 	this->setFont( parent->font() ) ;
 	this->setDefaultOpenMode() ;
 
+	m_ui->cbShareMountPoint->setChecked( utility::mountWithSharedMountPoint() ) ;
+
 	m_ui->PushButtonMountPointPath->setIcon( QIcon( ":/folder.png" ) ) ;
 
 	m_open_with_path = false ;
@@ -88,8 +92,6 @@ passwordDialog::passwordDialog( QTableWidget * table,
 	m_pluginMenu = new QMenu( this ) ;
 	m_pluginMenu->setFont( this->font() ) ;
 
-	m_ui->pushButtonPlugin->setIcon( QIcon( ":/module.png" ) ) ;
-
 	m_veraCryptVolume = utility::autoSetVolumeAsVeraCrypt( "zuluCrypt-gui" ) ;
 
 	if( m_veraCryptVolume ){
@@ -108,15 +110,18 @@ passwordDialog::passwordDialog( QTableWidget * table,
 	connect( m_ui->pushButtonPassPhraseFromFile,SIGNAL( clicked() ),this,SLOT( clickedPassPhraseFromFileButton() ) ) ;
 	connect( m_ui->OpenVolumePath,SIGNAL( textChanged( QString ) ),this,SLOT( mountPointPath( QString ) ) ) ;
 	connect( m_ui->checkBoxReadOnly,SIGNAL( stateChanged( int ) ),this,SLOT( cbStateChanged( int ) ) ) ;
-	connect( m_ui->pushButtonPlugin,SIGNAL( clicked() ),this,SLOT( pbPlugin() ) ) ;
 	connect( m_ui->pbKeyOption,SIGNAL( clicked() ),this,SLOT( pbKeyOption() ) ) ;
 	connect( m_ui->cbKeyType,SIGNAL( currentIndexChanged( int ) ),this,SLOT( cbActicated( int ) ) ) ;
 	connect( m_ui->cbVolumeType,SIGNAL( currentIndexChanged( int ) ),this,SLOT( cbVolumeType( int ) ) ) ;
 	connect( m_ui->checkBoxVisibleKey,SIGNAL( stateChanged( int ) ),this,SLOT( cbVisibleKeyStateChanged( int ) ) ) ;
 
+	connect( m_ui->cbShareMountPoint,&QCheckBox::stateChanged,[]( int s ){
+
+		utility::mountWithSharedMountPoint( s == Qt::Checked ) ;
+	} ) ;
+
 	m_ui->PushButtonMountPointPath->setVisible( false ) ;
 	m_ui->pushButtonPassPhraseFromFile->setVisible( false ) ;
-	m_ui->pushButtonPlugin->setVisible( false ) ;
 
 	m_veraCryptWarning.setWarningLabel( m_ui->veraCryptWarning ) ;
 
@@ -208,7 +213,7 @@ bool passwordDialog::eventFilter( QObject * watched,QEvent * event )
 void passwordDialog::pbPlugin()
 {
 	utility::createPlugInMenu( m_pluginMenu,tr( INTERNAL_WALLET ),
-				   tr( GNOME_WALLET ),tr( KWALLET ),true ) ;
+				   tr( GNOME_WALLET ),tr( KWALLET ),!utility::useZuluPolkit() ) ;
 
 	m_pluginMenu->addSeparator() ;
 
@@ -400,8 +405,6 @@ void passwordDialog::pluginOption()
 	m_ui->labelPassphrase->setText( tr( "Plugin Name" ) ) ;
 	m_ui->pushButtonPassPhraseFromFile->setIcon( QIcon( ":/keyfile.png" ) ) ;
 	m_ui->pbKeyOption->setIcon( QIcon( ":/module.png" ) ) ;
-	m_ui->pushButtonPlugin->setEnabled( true ) ;
-	m_ui->pushButtonPlugin->setToolTip( tr( "Select A Key Module" ) ) ;
 	m_ui->pbKeyOption->setEnabled( true ) ;
 	m_ui->PassPhraseField->setEnabled( false ) ;
 	m_ui->PassPhraseField->setText( INTERNAL_WALLET ) ;
@@ -413,9 +416,8 @@ void passwordDialog::passphraseOption()
 	m_ui->PassPhraseField->setEchoMode( QLineEdit::Password ) ;
 	m_ui->PassPhraseField->clear() ;
 	m_ui->pushButtonPassPhraseFromFile->setEnabled( false ) ;
-	m_ui->labelPassphrase->setText( tr( "Key" ) ) ;
+	m_ui->labelPassphrase->setText( tr( "Password" ) ) ;
 	m_ui->pushButtonPassPhraseFromFile->setIcon( QIcon( ":/passphrase.png" ) ) ;
-	m_ui->pushButtonPlugin->setEnabled( false ) ;
 	m_ui->pbKeyOption->setIcon( QIcon() ) ;
 	m_ui->pbKeyOption->setEnabled( false ) ;
 	m_ui->PassPhraseField->setFocus() ;
@@ -432,9 +434,7 @@ void passwordDialog::passphraseFromFileOption()
 	m_ui->pushButtonPassPhraseFromFile->setFocus() ;
 	m_ui->labelPassphrase->setText( tr( "KeyFile Path" ) ) ;
 	m_ui->pushButtonPassPhraseFromFile->setIcon( QIcon( ":/keyfile.png" ) ) ;
-	m_ui->pushButtonPlugin->setEnabled( false ) ;
 	m_ui->pbKeyOption->setIcon( QIcon( ":/keyfile.png" ) ) ;
-	m_ui->pushButtonPlugin->setEnabled( true ) ;
 	m_ui->pbKeyOption->setEnabled( true ) ;
 	m_ui->PassPhraseField->setEnabled( true ) ;
 }
@@ -588,7 +588,6 @@ void passwordDialog::disableAll()
 	m_ui->cbVolumeType->setEnabled( false ) ;
 	m_ui->labelVolumeType->setEnabled( false ) ;
 	m_ui->cbShareMountPoint->setEnabled( false ) ;
-	m_ui->pushButtonPlugin->setEnabled( false ) ;
 	m_ui->checkBoxReadOnly->setEnabled( false ) ;
 	m_ui->labelMoutPointPath->setEnabled( false ) ;
 	m_ui->labelPassphrase->setEnabled( false ) ;
@@ -616,7 +615,6 @@ void passwordDialog::enableAll()
 	m_ui->cbVolumeType->setEnabled( true ) ;
 	m_ui->labelVolumeType->setEnabled( true ) ;
 	m_ui->cbShareMountPoint->setEnabled( true ) ;
-	m_ui->pushButtonPlugin->setEnabled( true ) ;
 	m_ui->checkBoxReadOnly->setEnabled( true ) ;
 	m_ui->labelMoutPointPath->setEnabled( true ) ;
 	m_ui->labelPassphrase->setEnabled( true ) ;
@@ -641,17 +639,11 @@ void passwordDialog::enableAll()
 	if( m_ui->cbKeyType->currentIndex() == passwordDialog::key ){
 
 		m_ui->pushButtonPassPhraseFromFile->setEnabled( false ) ;
-		m_ui->pushButtonPlugin->setEnabled( false ) ;
 		m_ui->PassPhraseField->setEnabled( true ) ;
 	}else{
 		m_ui->pbKeyOption->setEnabled( true ) ;
 		m_ui->PassPhraseField->setEnabled( false ) ;
 	}
-
-	if( m_ui->cbKeyType->currentIndex() == passwordDialog::keyfile ){
-
-		m_ui->pushButtonPlugin->setEnabled( false ) ;
-	}
 }
 
 void passwordDialog::openVolume()
@@ -870,14 +862,16 @@ void passwordDialog::openVolume()
 
 		this->HideUI() ;
 	}else{
-		this->failed( r.exitCode() ) ;
+		this->failed( r ) ;
 
 		m_veraCryptWarning.hide() ;
 	}
 }
 
-void passwordDialog::failed( int r )
+void passwordDialog::failed( const utility::Task& e )
 {
+	int r = e.exitCode() ;
+
 	if( r == 12 && m_ui->cbKeyType->currentIndex() == passwordDialog::plugin ){
 		/*
 		 * A user cancelled the plugin
@@ -913,9 +907,11 @@ void passwordDialog::failed( int r )
 		case 22: msg.ShowUIOK( tr( "ERROR!" ),tr( "Insufficient privilege to open a system volume.\n\nConsult menu->help->permission for more informaion\n" ) ) ;					break ;
 		case 113:msg.ShowUIOK( tr( "ERROR!" ),tr( "A non supported device encountered,device is missing or permission denied\n\
 Possible reasons for getting the error are:\n1.Device path is invalid.\n2.The device has LVM or MDRAID signature" ) ) ;					break ;
-		default: msg.ShowUIOK( tr( "ERROR!" ),tr( "Unrecognized ERROR with status number %1 encountered" ).arg( r ) ) ;
+		default: msg.ShowUIOK( tr( "ERROR!" ),tr( "Error Code: %1\n--\nStdOut: %2\n--\nStdError: %3").arg( QString::number( e.exitCode() ),QString( e.stdError() ),QString( e.stdOut() ) ) ) ;
 	}
 
+	utility::debug() << e.stdOut() << "--" << e.stdError() ;
+
 	this->enableAll() ;
 
 	auto index = m_ui->cbKeyType->currentIndex() ;
