@@ -65,7 +65,7 @@
 #include "install_prefix.h"
 #include "utility.h"
 #include "executablesearchpaths.h"
-#include "networkAccessManager.hpp"
+#include "network_access_manager.hpp"
 #include "zuluPolkit.h"
 
 #include <sys/types.h>
@@ -103,6 +103,36 @@ static std::function< void() > _failed_to_connect_to_zulupolkit ;
 
 static QSettings * _settings ;
 
+static debugWindow * _debugWindow ;
+
+
+static void _post_backend_cmd( const QString& b )
+{
+	QString a = "***************************\n" ;
+	QString c = "\n***************************" ;
+
+	_debugWindow->UpdateOutPut( a + b + c,true ) ;
+}
+
+void utility::setDebugWindow( debugWindow * w )
+{
+	_debugWindow = w ;
+}
+
+std::unique_ptr< utility::RandomDataSource > utility::RandomDataSource::get( utility::RandomDataSource::types type )
+{
+	if( type == utility::RandomDataSource::types::urandom ){
+
+		return std::make_unique< utility::UrandomDataSource >() ;
+	}else{
+		return std::make_unique< utility::CRandDataSource >() ;
+	}
+}
+
+utility::RandomDataSource::~RandomDataSource()
+{
+}
+
 static QByteArray _json_command( const QByteArray& cookie,
 				 const QByteArray& password,
 				 const QString& exe,
@@ -164,7 +194,7 @@ static bool _connected( QLocalSocket& s )
 }
 
 static QByteArray _cookie ;
-static bool _polkit_support = AUTO_ENABLE_POLKIT_SUPPORT ;
+static bool _run_through_polkit ;
 
 ::Task::future< utility::Task >& utility::Task::run( const QString& exe,USEPOLKIT e )
 {
@@ -189,7 +219,7 @@ void utility::polkitFailedWarning( std::function< void() > e )
 void utility::Task::execute( const QString& exe,int waitTime,
 			     const QProcessEnvironment& env,
 			     const QByteArray& password,
-			     const std::function< void() >& f,
+			     std::function< void() > f,
 			     USEPOLKIT polkit )
 {
 	if( polkit == USEPOLKIT::True && utility::useZuluPolkit() ){
@@ -198,11 +228,13 @@ void utility::Task::execute( const QString& exe,int waitTime,
 
 		if( _connected( s ) ){
 
+			_post_backend_cmd( exe ) ;
+
 			s.write( _json_command( _cookie,password,exe ) ) ;
 
 			s.waitForBytesWritten() ;
 
-			s.waitForReadyRead() ;
+			s.waitForReadyRead( -1 ) ;
 
 			auto e = _json_result( s.readAll() ) ;
 
@@ -221,38 +253,15 @@ void utility::Task::execute( const QString& exe,int waitTime,
 			m_stdOut     = QObject::tr( "zuluCrypt: Failed To Establish Connection With zuluPolkit" ).toLatin1() ;
 		}
 	}else{
-		class Process : public QProcess{
-		public:
-			Process( const std::function< void() >& f ) : m_function( f )
-			{
-			}
-		protected:
-			void setupChildProcess()
-			{
-				m_function() ;
-			}
-		private:
-			const std::function< void() >& m_function ;
-		} p( f ) ;
+		_post_backend_cmd( exe ) ;
 
-		p.setProcessEnvironment( env ) ;
+		auto p = ::Task::process::run( exe,{},waitTime,password,env,std::move( f ) ).get() ;
 
-		p.start( exe ) ;
-
-		if( !password.isEmpty() ){
-
-			p.waitForStarted() ;
-
-			p.write( password + '\n' ) ;
-
-			p.closeWriteChannel() ;
-		}
-
-		m_finished   = p.waitForFinished( waitTime ) ;
-		m_exitCode   = p.exitCode() ;
-		m_exitStatus = p.exitStatus() ;
-		m_stdOut     = p.readAllStandardOutput() ;
-		m_stdError   = p.readAllStandardError() ;
+		m_finished   = p.finished() ;
+		m_exitCode   = p.exit_code() ;
+		m_exitStatus = p.exit_status() ;
+		m_stdOut     = p.std_out() ;
+		m_stdError   = p.std_error() ;
 	}
 }
 
@@ -303,11 +312,14 @@ static QString zuluPolkitExe()
 
 static ::Task::future< utility::Task >& _start_zulupolkit( const QString& e )
 {
-	QFile f( "/dev/urandom" ) ;
+	utility::UrandomDataSource randSource ;
 
-	f.open( QIODevice::ReadOnly ) ;
+	if( randSource.open() ){
 
-	_cookie = f.read( 16 ).toHex() ;
+		_cookie = randSource.getData( 16 ).toHex() ;
+	}else{
+		_cookie = utility::CRandDataSource().getData( 16 ).toHex() ;
+	}
 
 	return ::Task::run( [ = ]{
 
@@ -320,9 +332,26 @@ static ::Task::future< utility::Task >& _start_zulupolkit( const QString& e )
 	} ) ;
 }
 
-void utility::startHelperExecutable( QObject * obj,const QString& arg,const char * slot,const char * slot1 )
+static bool _enable_polkit_support( const QString& m )
 {
-	if( AUTO_ENABLE_POLKIT_SUPPORT ){
+	struct stat st ;
+
+	if( m == "zuluCrypt" ){
+
+		stat( ZULUCRYPTzuluCrypt,&st ) ;
+	}else{
+		stat( zuluMountPath,&st ) ;
+	}
+
+	bool s = st.st_mode & S_ISUID ;
+
+	return !s ;
+}
+
+void utility::startHelperExecutable( QObject * obj,const QString& arg,const QString& exe,
+				     const char * slot,const char * slot1 )
+{
+	if( _enable_polkit_support( exe ) ){
 
 		auto exe = zuluPolkitExe() ;
 
@@ -330,6 +359,8 @@ void utility::startHelperExecutable( QObject * obj,const QString& arg,const char
 
 			_start_zulupolkit( exe ).then( [ = ]( const utility::Task& e ){
 
+				_run_through_polkit = e.success() ;
+
 				QMetaObject::invokeMethod( obj,
 							   slot,
 							   Q_ARG( bool,e.success() ),
@@ -356,7 +387,7 @@ QString utility::helperSocketPath()
 
 bool utility::useZuluPolkit()
 {
-	return _polkit_support ;
+	return _run_through_polkit ;
 }
 
 bool utility::requireSystemPermissions( const QString& e,utility::background_thread thread )
@@ -398,7 +429,7 @@ bool utility::requireSystemPermissions( const QString& e,utility::background_thr
 
 bool utility::enablePolkit( utility::background_thread thread )
 {
-	if( _polkit_support ){
+	if( _run_through_polkit ){
 
 		return true ;
 	}
@@ -413,7 +444,7 @@ bool utility::enablePolkit( utility::background_thread thread )
 
 			if( _start_zulupolkit( exe ).get().success() ){
 
-				_polkit_support = true ;
+				_run_through_polkit = true ;
 
 				while( !utility::pathExists( socketPath ) ){
 
@@ -423,7 +454,7 @@ bool utility::enablePolkit( utility::background_thread thread )
 		}else{
 			if( _start_zulupolkit( exe ).await().success() ){
 
-				_polkit_support = true ;
+				_run_through_polkit = true ;
 
 				while( !utility::pathExists( socketPath ) ){
 
@@ -433,7 +464,7 @@ bool utility::enablePolkit( utility::background_thread thread )
 		}
 	}
 
-	return _polkit_support ;
+	return _run_through_polkit ;
 }
 
 void utility::quitHelper()
@@ -503,7 +534,7 @@ bool utility::reUseMountPoint()
 
 int utility::getUID()
 {
-	return getuid() ;
+	return static_cast< int >( getuid() ) ;
 }
 
 int utility::getUserID()
@@ -513,11 +544,11 @@ int utility::getUserID()
 
 int utility::getGUID( int uid )
 {
-	auto e = getpwuid( uid ) ;
+	auto e = getpwuid( static_cast< uid_t >( uid ) ) ;
 
 	if( e ){
 
-		return e->pw_gid ;
+		return static_cast< int >( e->pw_gid ) ;
 	}else{
 		return uid ;
 	}
@@ -540,7 +571,7 @@ QString utility::appendUserUID( const QString& e )
 
 static passwd * _getPassWd()
 {
-	return getpwuid( utility::getUserID() ) ;
+	return getpwuid( static_cast< uid_t >( utility::getUserID() ) ) ;
 }
 
 QString utility::userName()
@@ -550,7 +581,7 @@ QString utility::userName()
 
 QString utility::homePath()
 {
-	return getpwuid( utility::getUserID() )->pw_dir ;
+	return getpwuid( static_cast< uid_t >( utility::getUserID() ) )->pw_dir ;
 }
 
 static int _help()
@@ -604,7 +635,7 @@ void utility::keySend( const QString& path,const QByteArray& key )
 
 		if( handle ){
 
-			size_t size = key.size() ;
+			size_t size = static_cast< size_t >( key.size() ) ;
 
 			/*
 			 * ZULUCRYPT_KEYFILE_MAX_SIZE is defined in ../zuluCrypt-cli/constants.h
@@ -745,7 +776,7 @@ void utility::dropPrivileges( int uid )
 
 	if( uid != -1 ){
 
-		auto id = getpwuid( uid ) ;
+		auto id = getpwuid( static_cast< uid_t >( uid ) ) ;
 
 		if( id ){
 
@@ -754,10 +785,10 @@ void utility::dropPrivileges( int uid )
 			setenv( "USER",id->pw_name,1 ) ;
 		}
 
-		Q_UNUSED( setgid( uid ) ) ;
+		Q_UNUSED( setgid( static_cast< gid_t >( uid ) ) ) ;
 		Q_UNUSED( setgroups( 1,reinterpret_cast< const gid_t * >( &uid ) ) ) ;
-		Q_UNUSED( setegid( uid ) ) ;
-		Q_UNUSED( setuid( uid ) ) ;
+		Q_UNUSED( setegid( static_cast< gid_t >( uid ) ) ) ;
+		Q_UNUSED( setuid( static_cast< gid_t >( uid ) ) ) ;
 	}
 }
 
@@ -1056,6 +1087,16 @@ bool utility::pathPointsToAFolder( const QString& path )
 	}
 }
 
+bool utility::useDmCryptForRandomData()
+{
+	if( !_settings->contains( "UseDmCryptForRandomData" ) ){
+
+		_settings->setValue( "UseDmCryptForRandomData",false ) ;
+	}
+
+	return _settings->value( "UseDmCryptForRandomData" ).toBool() ;
+}
+
 QString utility::localizationLanguage( const QString& program )
 {
 	Q_UNUSED( program ) ;
@@ -1148,11 +1189,16 @@ bool utility::getOpenVolumeReadOnlyOption( const QString& app )
 
 QString utility::keyPath()
 {
-	QFile f( "/dev/urandom" ) ;
+	utility::UrandomDataSource randomSource ;
 
-	f.open( QIODevice::ReadOnly ) ;
+	QByteArray data ;
 
-	QByteArray data = f.read( 64 ) ;
+	if( randomSource.open() ){
+
+		data = randomSource.getData( 64 ) ;
+	}else{
+		data = utility::CRandDataSource().getData( 64 ) ;
+	}
 
 	QString a = utility::passwordSocketPath() ;
 
@@ -1181,12 +1227,17 @@ bool utility::eventFilter( QObject * gui,QObject * watched,QEvent * event,std::f
 
 QStringList utility::split( const QString& str,char token )
 {
+	if( str.isEmpty() ){
+
+		return {} ;
+	}
+
 	return str.split( token,QString::SkipEmptyParts ) ;
 }
 
 QStringList utility::split( const QByteArray& str,char token )
 {
-	return QString( str ).split( token,QString::SkipEmptyParts ) ;
+	return  utility::split( QString( str ),token ) ;
 }
 
 bool utility::mapperPathExists( const QString& path )
@@ -1294,7 +1345,7 @@ QString utility::hashPath( const QByteArray& p )
 
 	for( int i = 0 ; i < l ; i++ ){
 
-		hash += *( key + i ) ;
+		hash += static_cast< unsigned int >( *( key + i ) ) ;
 
 		hash += ( hash << 10 ) ;
 
@@ -1512,6 +1563,22 @@ QString utility::getVolumeID( const QString& id,bool expand )
 	return _partition_id_to_device_id( id,expand ) ;
 }
 
+
+bool utility::showWarningOnExtendingCoverFile()
+{
+	if( !_settings->contains( "ShowWarningOnExtendingCoverFile" ) ){
+
+		 utility::showWarningOnExtendingCoverFile( true ) ;
+	}
+
+	return _settings->value( "ShowWarningOnExtendingCoverFile" ).toBool() ;
+}
+
+void utility::showWarningOnExtendingCoverFile( bool e )
+{
+	_settings->setValue( "ShowWarningOnExtendingCoverFile",e ) ;
+}
+
 void utility::addToFavorite( const QString& dev,const QString& m_point )
 {
 	if( !( dev.isEmpty() || m_point.isEmpty() ) ){
@@ -1553,6 +1620,16 @@ QStringList utility::readFavorites()
 	}
 }
 
+bool utility::unMountVolumesOnLogout()
+{
+	if( !_settings->contains( "unMountVolumesOnLogout" ) ){
+
+		_settings->setValue( "unMountVolumesOnLogout",false ) ;
+	}
+
+	return _settings->value( "unMountVolumesOnLogout" ).toBool() ;
+}
+
 void utility::removeFavoriteEntry( const QString& e )
 {
 	auto s = utility::readFavorites() ;
@@ -1819,7 +1896,7 @@ bool utility::notRunningInMixedMode()
 
 bool utility::userBelongsToGroup( const char * groupname )
 {
-	auto user = getpwuid( utility::getUserID() ) ;
+	auto user = getpwuid( static_cast< gid_t >( utility::getUserID() ) ) ;
 
 	if( user != nullptr ){
 
@@ -2059,6 +2136,8 @@ void utility::setLocalizationLanguage( bool translate,QMenu * m,const QString& a
 
 void utility::languageMenu( QWidget * w,QMenu * m,QAction * ac,const char * app )
 {
+	Q_UNUSED( w ) ;
+
 	auto e = ac->text() ;
 
 	e.remove( "&" ) ;
@@ -2070,10 +2149,6 @@ void utility::languageMenu( QWidget * w,QMenu * m,QAction * ac,const char * app
 	_selectOption( m,e ) ;
 
 	return ;
-
-	DialogMsg msg( w ) ;
-
-	msg.ShowUIOK( QObject::tr( "INFO" ),QObject::tr( "Translation will be done the next time you restart." ) ) ;
 }
 
 QStringList utility::directoryList( const QString& e )
@@ -2210,6 +2285,36 @@ QString utility::powerOffCommand()
 	}
 }
 
+void utility::setDoNotMinimizeToTray( bool e )
+{
+	_settings->setValue( "doNotMinimizeToTray",e ) ;
+}
+
+bool utility::doNotMinimizeToTray()
+{
+	if( !_settings->contains( "doNotMinimizeToTray" ) ){
+
+		_settings->setValue( "doNotMinimizeToTray",false ) ;
+	}
+
+	return _settings->value( "doNotMinimizeToTray" ).toBool() ;
+}
+
+void utility::mountWithSharedMountPoint( bool e )
+{
+	_settings->setValue( "mountWithSharedMountPoint",e ) ;
+}
+
+bool utility::mountWithSharedMountPoint()
+{
+	if( !_settings->contains( "mountWithSharedMountPoint" ) ){
+
+		_settings->setValue( "mountWithSharedMountPoint",false ) ;
+	}
+
+	return _settings->value( "mountWithSharedMountPoint" ).toBool() ;
+}
+
 QString utility::prettyfySpaceUsage( quint64 s )
 {
 	auto _convert = [ & ]( const char * p,double q ){
@@ -2398,7 +2503,7 @@ static inline bool _terminalEchoOff( struct termios * old,struct termios * curre
 	}
 
 	*current = *old;
-	current->c_lflag &= ~ECHO;
+	current->c_lflag &= static_cast< unsigned int >( ~ECHO ) ;
 
 	if( tcsetattr( 1,TCSAFLUSH,current ) != 0 ){
 
@@ -2532,3 +2637,242 @@ QString utility::failedToStartzuluPolkit()
 {
 	return QObject::tr( "Failed To Start Helper Application.\n\n\"org.zulucrypt.zulupolkit.policy\" polkit file is misconfigured,\nzuluPolkit executable could not be found\n or pkexec failed to start zuluPolkit." ) ;
 }
+
+static utility::result< int > _convert_string_to_version( const QString& e )
+{
+	auto _convert = []( const QString& e )->utility::result< int >{
+
+		bool ok ;
+
+		auto s = e.toInt( &ok ) ;
+
+		if( ok ){
+
+			return s  ;
+		}else{
+			return {} ;
+		}
+	} ;
+
+	auto s = utility::split( e,'.' ) ;
+
+	auto components = s.size() ;
+
+	int major = 1000000 ;
+	int minor = 1000 ;
+	int patch = 1 ;
+
+	if( components == 1 ){
+
+		auto a = _convert( s.first() ) ;
+
+		if( a ){
+
+			return major * a.value() ;
+		}
+
+	}else if( components == 2 ){
+
+		auto a = _convert( s.at( 0 ) ) ;
+		auto b = _convert( s.at( 1 ) ) ;
+
+		if( a && b ){
+
+			return major * a.value() + minor * b.value() ;
+		}
+
+	}else if( components == 3 ){
+
+		auto a = _convert( s.at( 0 ) ) ;
+		auto b = _convert( s.at( 1 ) ) ;
+		auto c = _convert( s.at( 2 ) ) ;
+
+		if( a && b && c ){
+
+			return major * a.value() + minor * b.value() + patch * c.value() ;
+		}
+	}
+
+	return {} ;
+}
+
+static utility::result< QString > _installed_version( const QString& backend )
+{
+	auto _remove_junk = []( QString e ){
+
+		e.replace( "v","" ).replace( ";","" ) ;
+
+		QString m ;
+
+		for( int s = 0 ; s < e.size() ; s++ ){
+
+			auto n = e.at( s ) ;
+
+			if( n == '.' || ( n >= '0' && n <= '9' ) ){
+
+				m += n ;
+			}else{
+				break ;
+			}
+		}
+
+		return m ;
+	} ;
+
+	auto exe = utility::executableFullPath( backend ) ;
+
+	if( exe.isEmpty() ){
+
+		return {} ;
+	}
+
+	auto cmd = [ & ](){
+
+		if( backend == "securefs" ){
+
+			return backend + " version" ;
+		}else{
+			return backend + " --version" ;
+		}
+	}() ;
+
+	auto s = utility::systemEnvironment() ;
+
+	auto r = [ & ](){
+
+		if( backend == "encfs" ){
+
+			return QString( ::Task::process::run( cmd,{},-1,{},s ).get().std_error() ) ;
+		}else{
+			return QString( ::Task::process::run( cmd,{},-1,{},s ).get().std_out() ) ;
+		}
+	}() ;
+
+	if( r.isEmpty() ){
+
+		return {} ;
+	}
+
+	auto m = utility::split( utility::split( r,'\n' ).first(),' ' ) ;
+
+	if( utility::equalsAtleastOne( backend,"cryfs","encfs","sshfs" ) ){
+
+		if( m.size() >= 3 ){
+
+			return _remove_junk( m.at( 2 ) ) ;
+		}
+
+	}else if( utility::equalsAtleastOne( backend,"gocryptfs","securefs","ecryptfs-simple" ) ){
+
+		if( m.size() >= 2 ){
+
+			return _remove_junk( m.at( 1 ) ) ;
+		}
+	}
+
+	return {} ;
+}
+
+::Task::future< utility::result< QString > >& utility::backEndInstalledVersion( const QString& backend )
+{
+	return ::Task::run( _installed_version,backend ) ;
+}
+
+static utility::result< int > _installedVersion( const QString& backend )
+{
+	auto s = utility::backEndInstalledVersion( backend ).get() ;
+
+	if( s && !s.value().isEmpty() ){
+
+		return _convert_string_to_version( s.value() ) ;
+	}else{
+		return {} ;
+	}
+}
+
+template< typename Function >
+::Task::future< utility::result< bool > >& _compare_versions( const QString& backend,
+							     const QString& version,
+							     Function compare )
+{
+	return ::Task::run( [ = ]()->utility::result< bool >{
+
+		auto installed = _installedVersion( backend ) ;
+		auto guard_version = _convert_string_to_version( version ) ;
+
+		if( installed && guard_version ){
+
+			return compare( installed.value(),guard_version.value() ) ;
+		}else{
+			return {} ;
+		}
+	} ) ;
+}
+
+::Task::future< utility::result< bool > >& utility::backendIsGreaterOrEqualTo( const QString& backend,
+									       const QString& version )
+{
+	return _compare_versions( backend,version,std::greater_equal<int>() ) ;
+}
+
+::Task::future< utility::result< bool > >& utility::backendIsLessThan( const QString& backend,
+								       const QString& version )
+{
+	return _compare_versions( backend,version,std::less<int>() ) ;
+}
+
+utility::UrandomDataSource::UrandomDataSource() :
+	m_file( "/dev/urandom" )
+{
+}
+
+bool utility::UrandomDataSource::open()
+{
+	m_file.open( QIODevice::ReadOnly ) ;
+
+	return m_file.isOpen() ;
+}
+
+qint64 utility::UrandomDataSource::getData( char * data,qint64 size )
+{
+	return m_file.read( data,size ) ;
+}
+
+QByteArray utility::UrandomDataSource::getData( qint64 size )
+{
+	return m_file.read( size ) ;
+}
+
+utility::CRandDataSource::CRandDataSource()
+{
+}
+
+bool utility::CRandDataSource::open()
+{
+	return true ;
+}
+
+qint64 utility::CRandDataSource::getData( char * data,qint64 size )
+{
+	time_t t ;
+
+	srand( static_cast< unsigned int >( time( &t ) ) ) ;
+
+	for( int i = 0 ; i < size ; i++ ){
+
+	   *( data + i ) = static_cast< char >( rand() ) ;
+	}
+
+	return size ;
+}
+
+QByteArray utility::CRandDataSource::getData( qint64 size )
+{
+	QByteArray data ;
+
+	data.resize( static_cast< int >( size ) ) ;
+
+	utility::CRandDataSource().getData( data.data(),size ) ;
+
+	return data ;
+}
