@@ -44,6 +44,7 @@
 #include <memory>
 #include <array>
 #include <utility>
+#include <chrono>
 #include <QtNetwork/QLocalSocket>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -53,8 +54,9 @@
 
 #include <blkid/blkid.h>
 
-#include "task.h"
+#include "task.hpp"
 #include "lxqt_wallet.h"
+#include "debugwindow.h"
 
 #include <QObject>
 #include <QLabel>
@@ -67,6 +69,53 @@
 
 namespace utility
 {
+template< typename T >
+class result
+{
+public:
+		result()
+		{
+		}
+		result( T e ) : m_valid( true ),m_value( std::move( e ) )
+		{
+		}
+		T * operator->()
+		{
+			return &m_value ;
+		}
+		const T * operator->() const
+		{
+			return &m_value ;
+		}
+		T& operator*()
+		{
+			return m_value ;
+		}
+		const T& operator*() const
+		{
+			return m_value ;
+		}
+		operator bool()
+		{
+			return m_valid ;
+		}
+		bool has_value() const
+		{
+			return m_valid ;
+		}
+		T& value()
+		{
+			return m_value ;
+		}
+		const T& value() const
+		{
+			return m_value ;
+		}
+	private:
+		bool m_valid = false ;
+		T m_value ;
+	} ;
+
 	class raii
 	{
 	public:
@@ -91,6 +140,36 @@ namespace utility
 }
 namespace utility
 {
+	struct RandomDataSource
+	{
+		enum class types{ urandom,crand };
+		static std::unique_ptr< RandomDataSource > get( types = types::urandom ) ;
+		virtual ~RandomDataSource() ;
+		virtual bool open() = 0 ;
+		virtual qint64 getData( char * data,qint64 size ) = 0 ;
+		virtual QByteArray getData( qint64 size ) = 0 ;
+	} ;
+
+	class UrandomDataSource : public RandomDataSource
+	{
+	public:
+		UrandomDataSource() ;
+		bool open() override ;
+		qint64 getData( char * data,qint64 size ) override ;
+		QByteArray getData( qint64 size ) override ;
+	private:
+		QFile m_file;
+	} ;
+
+	class CRandDataSource : public RandomDataSource
+	{
+	public:
+		CRandDataSource() ;
+		bool open() override ;
+		qint64 getData( char * data,qint64 size ) override ;
+		QByteArray getData( qint64 size ) override ;
+	} ;
+
 	class debug
 	{
 	public:
@@ -293,6 +372,15 @@ namespace utility
 	void setSettingsObject( QSettings * ) ;
 	QSettings& settingsObject() ;
 
+	::Task::future< utility::result< QString > >& backEndInstalledVersion( const QString& backend ) ;
+
+	::Task::future< utility::result< bool > >& backendIsLessThan( const QString& backend,
+								      const QString& version ) ;
+
+	::Task::future< utility::result< bool > >& backendIsGreaterOrEqualTo( const QString& backend,
+									      const QString& version ) ;
+
+	bool useDmCryptForRandomData() ;
 	void setDefaultEnvironment() ;
 	QString passwordSocketPath() ;
 	QString socketPath() ;
@@ -308,8 +396,9 @@ namespace utility
 	bool pathExists( const QString& ) ;
 	bool canCreateFile( const QString& ) ;
 	bool useZuluPolkit( void ) ;
-	void startHelperExecutable( QObject *,const QString&,const char *,const char * ) ;
+	void startHelperExecutable( QObject *,const QString&,const QString&,const char *,const char * ) ;
 	void dropPrivileges( int = -1 ) ;
+	void setDebugWindow( debugWindow * w ) ;
 	QString fileManager( void ) ;
 	void setFileManager( const QString& ) ;
 	QString failedToStartzuluPolkit() ;
@@ -354,6 +443,10 @@ namespace utility
 	void licenseInfo( QWidget * ) ;
 	void showTrayIcon( QAction *,QObject * obj,bool = true ) ;
 	void trayProperty( QSystemTrayIcon *,bool = true ) ;
+	void setDoNotMinimizeToTray( bool ) ;
+	bool doNotMinimizeToTray() ;
+	bool mountWithSharedMountPoint() ;
+	void mountWithSharedMountPoint( bool ) ;
 	bool platformIsLinux( void ) ;
 	bool platformIsOSX( void ) ;
 	QStringList executableSearchPaths( void ) ;
@@ -368,6 +461,9 @@ namespace utility
 
 	QString executableFullPath( const QString& ) ;
 
+	bool showWarningOnExtendingCoverFile() ;
+	void showWarningOnExtendingCoverFile( bool ) ;
+
 	bool reUseMountPointPath( void ) ;
 	bool reUseMountPoint( void ) ;
 
@@ -382,6 +478,8 @@ namespace utility
 	void setLocalizationLanguage( bool translate,QMenu * ac,const QString& ) ;
 	void languageMenu( QWidget *,QMenu *,QAction *,const char * ) ;
 
+	bool unMountVolumesOnLogout( void ) ;
+
 	using array_t = std::array< int,10 > ;
 
 	utility::array_t getWindowDimensions( const QString& application ) ;
@@ -721,7 +819,7 @@ namespace utility
 		}	
 	private:
 		void execute( const QString& exe,int waitTime,const QProcessEnvironment& env,
-			      const QByteArray& password,const std::function< void() >& f,USEPOLKIT ) ;
+			      const QByteArray& password,std::function< void() > f,USEPOLKIT ) ;
 		QByteArray m_stdOut ;
 		QByteArray m_stdError ;
 
@@ -824,4 +922,43 @@ namespace utility
 	}
 }
 
+namespace utility {
+	class duration{
+	public:
+		duration( long miliseconds ) : m_milliseconds( miliseconds )
+		{
+			this->reset() ;
+		}
+		bool passed()
+		{
+			auto now = std::chrono::high_resolution_clock::now() ;
+
+			auto seconds = std::chrono::duration_cast< std::chrono::milliseconds >( now - m_start_time ).count() ;
+
+			if( seconds >= m_milliseconds ){
+
+				this->reset() ;
+
+				return true ;
+			}else{
+				return false ;
+			}
+		}
+		template< typename Function >
+		void passed( Function function )
+		{
+			if( this->passed() ){
+
+				function() ;
+			}
+		}
+	private:
+		void reset()
+		{
+			m_start_time = std::chrono::high_resolution_clock::now() ;
+		}
+		long m_milliseconds ;
+		decltype( std::chrono::high_resolution_clock::now() ) m_start_time ;
+	};
+}
 #endif // MISCFUNCTIONS_H
