@@ -26,7 +26,7 @@
 #include "tablewidget.h"
 #include "walletconfiginput.h"
 #include "utility.h"
-#include "task.h"
+#include "task.hpp"
 
 #include <Qt>
 #include <QTableWidget>
@@ -91,7 +91,7 @@ void walletconfig::itemClicked_0( QTableWidgetItem * item )
 
 	m_volumeID = m_ui->tableWidget->item( m_row,0 )->text() ;
 
-	int r = msg.ShowUIYesNo( tr( "WARNING!" ),tr( "Are you sure you want to delete a volume with an id of \"%1\"?" ).arg( m_volumeID ) ) ;
+	auto r = msg.ShowUIYesNo( tr( "WARNING!" ),tr( "Are you sure you want to delete a volume with an id of \"%1\"?" ).arg( m_volumeID ) ) ;
 
 	if( r == QMessageBox::Yes ){
 
