@@ -44,6 +44,7 @@
 #include <QCloseEvent>
 #include <QUrl>
 #include <QMimeData>
+#include <QShortcut>
 
 #include "../zuluCrypt-cli/constants.h"
 #include "../zuluMount-gui/oneinstance.h"
@@ -65,12 +66,13 @@
 #include "walletconfig.h"
 #include "tablewidget.h"
 #include "utility.h"
-#include "task.h"
+#include "task.hpp"
 #include "about.h"
 #include "help.h"
-
+#include "createvolumeinexistingfile.h"
 #include "veracrypt_support.h"
 #include "pdf_path.h"
+#include "warnwhenextendingcontainerfile.h"
 
 #include "checkforupdates.h"
 
@@ -79,8 +81,15 @@
 zuluCrypt::zuluCrypt( QWidget * parent ) :
 	QMainWindow( parent ),
 	m_mountInfo( this,false,[ this ](){ this->quitApplication() ; } ),
-	m_checkForUpdates( this )
+	m_checkForUpdates( this ),
+	m_signalHandler( this )
 {
+	m_signalHandler.setAction( [ this ]( systemSignalHandler::signal s ){
+
+		Q_UNUSED( s ) ;
+
+		this->emergencyQuitApplication() ;
+	} ) ;
 }
 
 void zuluCrypt::setLocalizationLanguage( bool translate )
@@ -138,7 +147,7 @@ void zuluCrypt::updateVolumeList( const QString& volume )
 {
 	m_ui->tableWidget->setEnabled( false ) ;
 
-	emit updateVolumeListSignal( volume,Task::await< QString >( [](){
+	emit updateVolumeListSignal( volume,Task::await( [](){
 
 		utility::Task::waitForOneSecond() ;
 
@@ -196,16 +205,6 @@ void zuluCrypt::updateVolumeList( QString volume,QString r )
 
 void zuluCrypt::initKeyCombo()
 {
-	this->addAction( [ this ](){
-
-		auto ac = new QAction( this ) ;
-
-		ac->setShortcuts( { Qt::Key_Menu,Qt::CTRL + Qt::Key_M } ) ;
-
-		connect( ac,SIGNAL( triggered() ),this,SLOT( menuKeyPressed() ) ) ;
-
-		return ac ;
-	}() ) ;
 }
 
 void zuluCrypt::initFont()
@@ -263,6 +262,7 @@ void zuluCrypt::start()
 			       utility::cmdArgumentValue( l,"-d" ),
 			       [ this ]( const QString& e ){ utility::startHelperExecutable( this,
 											     e,
+											     "zuluCrypt",
 											     "helperStarted",
 											     "closeApplication" ) ; },
 			       [ this ]( int s ){ this->closeApplication( s ) ;	},
@@ -309,15 +309,6 @@ void zuluCrypt::setupUIElements()
 
 	m_trayIcon.setParent( this ) ;
 
-	auto trayMenu = new QMenu( this ) ;
-
-	trayMenu->setFont( this->font() ) ;
-
-	trayMenu->addAction( tr( "Show/Hide" ),this,SLOT( showTrayGUI() ) ) ;
-	trayMenu->addAction( tr( "Quit" ),this,SLOT( closeApplication() ) ) ;
-
-	m_trayIcon.setContextMenu( trayMenu ) ;
-
 	const auto f = utility::getWindowDimensions( "zuluCrypt" ) ;
 
 	const auto e = f.data() ;
@@ -354,6 +345,20 @@ void zuluCrypt::setupConnections()
 {
 	m_ui->tableWidget->setMouseTracking( true ) ;
 
+	m_ui->tableWidget->setContextMenuPolicy( Qt::CustomContextMenu ) ;
+
+	connect( m_ui->tableWidget,&QTableWidget::customContextMenuRequested,[ this ]( QPoint s ){
+
+		Q_UNUSED( s ) ;
+
+		auto item = m_ui->tableWidget->currentItem() ;
+
+		if( item ){
+
+			this->itemClicked( item,QCursor::pos() ) ;
+		}
+	} ) ;
+
 	connect( this,SIGNAL( updateVolumeListSignal( QString,QString ) ),this,SLOT( updateVolumeList( QString,QString ) ),Qt::QueuedConnection ) ;
 
 	connect( m_ui->tableWidget,SIGNAL( itemEntered( QTableWidgetItem * ) ),this,SLOT( itemEntered( QTableWidgetItem * ) ) ) ;
@@ -361,6 +366,7 @@ void zuluCrypt::setupConnections()
 	connect( m_ui->actionPartitionOpen,SIGNAL( triggered() ),this,SLOT( ShowOpenPartition() ) ) ;
 	connect( m_ui->actionFileOpen,SIGNAL( triggered() ),this,SLOT( ShowPasswordDialog() ) ) ;
 	connect( m_ui->actionFileCreate,SIGNAL( triggered() ),this,SLOT( ShowCreateFile() ) ) ;
+	connect( m_ui->actionEncrypted_Container_In_An_Existing_FIle,SIGNAL( triggered() ),this,SLOT( createVolumeInExistingFile() ) ) ;
 	connect( m_ui->actionManage_names,SIGNAL( triggered() ),this,SLOT( ShowFavoritesEntries() ) ) ;
 	connect( m_ui->tableWidget,SIGNAL( currentItemChanged( QTableWidgetItem *,QTableWidgetItem * ) ),
 		 this,SLOT( currentItemChanged( QTableWidgetItem *,QTableWidgetItem * ) ) ) ;
@@ -394,6 +400,16 @@ void zuluCrypt::setupConnections()
 
 	connect( this,SIGNAL( closeVolume( QTableWidgetItem *,int ) ),this,SLOT( closeAll( QTableWidgetItem *,int ) ) ) ;
 
+	m_ui->actionDo_not_minimize_to_tray->setChecked( utility::doNotMinimizeToTray() ) ;
+
+	connect( m_ui->actionDo_not_minimize_to_tray,&QAction::triggered,[ this ](){
+
+		auto s = !utility::doNotMinimizeToTray() ;
+
+		m_ui->actionDo_not_minimize_to_tray->setChecked( s ) ;
+		utility::setDoNotMinimizeToTray( s ) ;
+	} ) ;
+
 	m_autoOpenMountPoint = utility::autoOpenFolderOnMount( "zuluCrypt-gui" ) ;
 
 	m_ui->actionAuto_Open_Mount_Point->setCheckable( true ) ;
@@ -434,7 +450,33 @@ void zuluCrypt::setupConnections()
 	m_ui->actionVeracrypt_container_in_a_file->setEnabled( true ) ;
 	m_ui->actionVeracrypt_container_in_a_partition->setEnabled( true ) ;
 
+	connect( m_ui->actionShow_Debug_Window,&QAction::triggered,[ this ](){
+
+		m_debugWindow.Show() ;
+	} ) ;
+
 	this->setAcceptDrops( true ) ;
+	this->updateTrayContextMenu() ;
+}
+
+void zuluCrypt::updateTrayContextMenu()
+{
+	m_trayIconMenu.clear() ;
+
+	m_trayIconMenu.addMenu( m_ui->menu_zc ) ;
+
+	m_trayIconMenu.addMenu( m_ui->menuOpen ) ;
+
+	m_trayIconMenu.addMenu( m_ui->menuCreate ) ;
+
+	//m_trayIconMenu.addMenu( m_ui->menuFavorites ) ;
+
+	m_trayIconMenu.setFont( this->font() ) ;
+
+	m_trayIconMenu.addAction( tr( "Show/Hide" ),this,SLOT( showTrayGUI() ) ) ;
+	m_trayIconMenu.addAction( tr( "Quit" ),this,SLOT( closeApplication() ) ) ;
+
+	m_trayIcon.setContextMenu( &m_trayIconMenu ) ;
 }
 
 void zuluCrypt::showTrayGUI()
@@ -497,6 +539,14 @@ void zuluCrypt::manageVolumesInKDEWallet()
 	_walletconfig( this,m_secrets,LXQt::Wallet::BackEnd::kwallet ) ;
 }
 
+void zuluCrypt::createVolumeInExistingFile()
+{
+	warnWhenExtendingContainerFile::Show( this,[ this ](){
+
+		createVolumeInExistingFIle::instance( this ) ;
+	} ) ;
+}
+
 void zuluCrypt::failedToOpenWallet()
 {
 	//DialogMsg msg( this ) ;
@@ -539,6 +589,46 @@ void zuluCrypt::currentItemChanged( QTableWidgetItem * current,QTableWidgetItem
 	}
 }
 
+void zuluCrypt::emergencyQuitApplication()
+{
+	this->hide() ;
+
+	m_mountInfo.announceEvents( false ) ;
+
+	Task::await( [ this ](){
+
+		auto table = m_ui->tableWidget ;
+
+		auto volumeCount = table->rowCount() ;
+
+		if( volumeCount > 0 ){
+
+			QVector< QTableWidgetItem * > tableItems( volumeCount ) ;
+
+			auto it = tableItems.data() ;
+
+			for( int i = 0 ; i < volumeCount ; i++ ){
+
+				*( it + i ) = table->item( i,0 ) ;
+			}
+
+			auto exe = utility::appendUserUID( "%1 -q -d \"%2\"" ) ;
+
+			for( int i = tableItems.count() - 1 ; i >= 0 ; i-- ){
+
+				auto e = *( it + i ) ;
+
+				auto device = e->text().replace( "\"","\"\"\"" ) ;
+
+				utility::Task( exe.arg( ZULUCRYPTzuluCrypt,device ) ) ;
+			}
+		}
+
+	} ) ;
+
+	this->quitApplication() ;
+}
+
 void zuluCrypt::closeAllVolumes()
 {
 	m_ui->tableWidget->setEnabled( false ) ;
@@ -618,7 +708,13 @@ void zuluCrypt::closeEvent( QCloseEvent * e )
 {
 	e->ignore() ;
 
-	if( m_trayIcon.isVisible() ){
+	if( utility::doNotMinimizeToTray() ){
+
+		this->hide() ;
+
+		this->closeApplication() ;
+
+	}else if( m_trayIcon.isVisible() ){
 
 		this->hide() ;
 	}else{
@@ -855,6 +951,7 @@ void zuluCrypt::favClicked( QAction * ac )
 void zuluCrypt::readFavorites()
 {
 	utility::readFavorites( m_ui->menuFavorites,false,false ) ;
+	this->updateTrayContextMenu() ;
 }
 
 void zuluCrypt::addToFavorite()
@@ -909,12 +1006,16 @@ void zuluCrypt::itemClicked( QTableWidgetItem * it )
 	this->itemClicked( it,true ) ;
 }
 
-void zuluCrypt::itemClicked( QTableWidgetItem * item,bool clicked )
+void zuluCrypt::itemClicked( QTableWidgetItem * item,QPoint point )
 {
 	QMenu m ;
 
 	m.setFont( this->font() ) ;
 
+	auto m_point = item->tableWidget()->item( item->row(),1 )->text() ;
+
+	m_sharedMountPoint = utility::sharedMountPointPath( m_point ) ;
+
 	if( m_sharedMountPoint.isEmpty() ){
 
 		connect( m.addAction( tr( "Open Folder" ) ) ,SIGNAL( triggered() ),this,SLOT( openFolder() ) ) ;
@@ -927,14 +1028,9 @@ void zuluCrypt::itemClicked( QTableWidgetItem * item,bool clicked )
 
 	m.addSeparator() ;
 
-	connect( m.addAction( tr( "Properties" ) ) ,SIGNAL( triggered() ),this,SLOT( volume_property() ) ) ;
-
-	m.addSeparator() ;
-
-	auto m_point = item->tableWidget()->item( item->row(),1 )->text() ;
-
-	m_sharedMountPoint = utility::sharedMountPointPath( m_point ) ;
-
+	auto ac = m.addAction( tr( "Properties" ) ) ;
+	//ac->setEnabled( !m_ui->tableWidget->item( item->row(),2 )->text().startsWith( "bitlocker" ) ) ;
+	connect( ac,SIGNAL( triggered() ),this,SLOT( volume_property() ) ) ;
 
 	m.addSeparator() ;
 
@@ -954,18 +1050,22 @@ void zuluCrypt::itemClicked( QTableWidgetItem * item,bool clicked )
 
 	auto volume_id = m_ui->tableWidget->item( item->row(),0 )->text() + "\t" ;
 
-	QFile f( utility::homePath() + "/.zuluCrypt/favorites" ) ;
+	auto has_favorite = [ & ](){
 
-	decltype( f.readAll() ) data ;
+		for( const auto& it : utility::readFavorites() ){
 
-	if( f.open( QIODevice::ReadOnly ) ){
+			if( it.startsWith( volume_id ) ){
 
-		data = f.readAll() ;
-	}
+				return true ;
+			}
+		}
 
-	auto ac = m.addAction( tr( "Add To Favorite" ) ) ;
+		return false ;
+	}() ;
 
-	if( data.contains( volume_id.toLatin1() ) ){
+	ac = m.addAction( tr( "Add To Favorite" ) ) ;
+
+	if( has_favorite ){
 
 		ac->setEnabled( false ) ;
 	}else{
@@ -977,16 +1077,22 @@ void zuluCrypt::itemClicked( QTableWidgetItem * item,bool clicked )
 
 	connect( m.addAction( tr( "Unmount" ) ),SIGNAL( triggered() ),this,SLOT( close() ) ) ;
 
+	m.addSeparator() ;
+	m.addAction( tr( "Cancel" ) ) ;
+
+	m.exec( point ) ;
+}
+
+void zuluCrypt::itemClicked( QTableWidgetItem * item,bool clicked )
+{
 	if( clicked ){
 
-		m.exec( QCursor::pos() ) ;
+		this->itemClicked( item,QCursor::pos() ) ;
 	}else{
 		int x = m_ui->tableWidget->columnWidth( 0 ) ;
 		int y = m_ui->tableWidget->rowHeight( item->row() ) * item->row() + 20 ;
 
-		m.addSeparator() ;
-		m.addAction( tr( "Cancel" ) ) ;
-		m.exec( m_ui->tableWidget->mapToGlobal( QPoint( x,y ) ) ) ;
+		this->itemClicked( item,m_ui->tableWidget->mapToGlobal( QPoint( x,y ) ) ) ;
 	}
 }
 
@@ -1045,12 +1151,10 @@ void zuluCrypt::close()
 
 	auto path = m_ui->tableWidget->item( item->row(),0 )->text().replace( "\"","\"\"\"" ) ;
 
-	auto r = Task::await<int>( [ this,item,path ](){
+	auto r = Task::await( [ path ](){
 
 		auto exe = utility::appendUserUID( "%1 -q -d \"%2\"" ).arg( ZULUCRYPTzuluCrypt,path ) ;
 
-		//utility::Task::waitForOneSecond() ; //for UI effect
-
 		return utility::Task( exe ).exitCode() ;
 	} ) ;
 
