@@ -30,6 +30,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <unistd.h>
+
 #include <grp.h>
 
 static void _seteuid( uid_t uid )
@@ -182,6 +183,17 @@ static int _zuluMountMountedList( uid_t uid )
 	return zuluMountPrintVolumesProperties( uid ) ;
 }
 
+static string_t _zuluCryptGetFileSystemFromDevice( const char * device )
+{
+	string_t st ;
+
+	zuluCryptSecurityGainElevatedPrivileges() ;
+	st = zuluCryptGetFileSystemFromDevice( device ) ;
+	zuluCryptSecurityDropElevatedPrivileges() ;
+
+	return st ;
+}
+
 static int _zuluPartitionHasCryptoFs( const char * device )
 {
 	int r ;
@@ -324,20 +336,35 @@ static int _zuluMountExe( ARGS * args )
 	const char * offset = args->offset ;
 	size_t       uid    = args->uid    ;
 
+	string_t st ;
+
+	int r ;
 
 	if( StringsAreEqual( action,"-L" ) ){
 
 		return zuluMountPrintDeviceProperties( device,uuid,uid ) ;
 	}
+
 	if( StringsAreEqual( action,"-s" ) ){
 
-		if( offset != NULL || _zuluPartitionHasCryptoFs( device ) ){
+		st = _zuluCryptGetFileSystemFromDevice( device ) ;
 
-			return zuluMountVolumeStatus( device,uuid,uid ) ;
+		if( StringContains( st,zuluCryptBitLockerType() ) ){
+
+			r = zuluMountPrintBitLockerProperties( device,uid ) ;
+
+		}else if( offset != NULL || st == StringVoid || StringStartsWith( st,"crypto_" ) ){
+
+			r = zuluMountVolumeStatus( device,uuid,uid ) ;
 		}else{
-			return zuluMountUnEncryptedVolumeStatus( device ) ;
+			r = zuluMountUnEncryptedVolumeStatus( device,NULL,NULL ) ;
 		}
+
+		StringDelete( &st ) ;
+
+		return r ;
 	}
+
 	if( StringsAreEqual( action,"-m" ) ){
 
 		if( offset != NULL || _zuluPartitionHasCryptoFs( device ) ){
@@ -705,7 +732,7 @@ int main( int argc,char * argv[] )
 	}
 	if( StringsAreEqual( args.action,"-E" ) ){
 
-		return _zuluExit_2(  zuluMountprintAListOfMountedVolumes(),stl,stx,NULL ) ;
+		return _zuluExit_2(  zuluMountprintAListOfMountedVolumes( uid ),stl,stx,NULL ) ;
 	}
 	if( StringsAreEqual( args.action,"-c" ) ){
 
