@@ -31,7 +31,7 @@
 
 #include "bin_path.h"
 #include "../zuluCrypt-gui/dialogmsg.h"
-#include "task.h"
+#include "task.hpp"
 #include "../zuluCrypt-cli/constants.h"
 #include "plugin_path.h"
 #include "../zuluCrypt-gui/utility.h"
@@ -62,6 +62,9 @@ keyDialog::keyDialog( QWidget * parent,
 	m_success( std::move( q ) )
 {
 	m_ui->setupUi( this ) ;
+
+	m_ui->lineEditKey->setMaxLength( 32767 ) ;
+
 	m_ui->checkBoxShareMountPoint->setToolTip( utility::shareMountPointToolTip() ) ;
 	m_table = table ;
 	m_path = e.volumeName() ;
@@ -79,6 +82,8 @@ keyDialog::keyDialog( QWidget * parent,
 
 	m_ui->checkBoxVisibleKey->setToolTip( tr( "Check This Box To Make Password Visible" ) ) ;
 
+	m_ui->checkBoxShareMountPoint->setChecked( utility::mountWithSharedMountPoint() ) ;
+
 	this->setWindowTitle( msg ) ;
 
 	m_ui->lineEditMountPoint->setText( m_path ) ;
@@ -118,6 +123,11 @@ keyDialog::keyDialog( QWidget * parent,
 
 	m_ui->pbOpenMountPoint->setVisible( false ) ;
 
+	connect( m_ui->checkBoxShareMountPoint,&QCheckBox::stateChanged,[]( int s ){
+
+		utility::mountWithSharedMountPoint( s == Qt::Checked ) ;
+	} ) ;
+
 	const auto& m = e.mountPoint() ;
 
 	if( m.isEmpty() || m == "Nil" ){
@@ -365,7 +375,7 @@ void keyDialog::pbkeyOption()
 void keyDialog::Plugin()
 {
 	utility::createPlugInMenu( m_menu,tr( INTERNAL_WALLET ),
-				   tr( GNOME_WALLET ),tr( KWALLET ),!m_encryptedFolder ) ;
+				   tr( GNOME_WALLET ),tr( KWALLET ),!m_encryptedFolder || !utility::useZuluPolkit() ) ;
 
 	m_menu->setFont( this->font() ) ;
 
