@@ -39,6 +39,7 @@
 #include <QTranslator>
 #include <QMimeData>
 #include <QFile>
+#include <QShortcut>
 
 #include <utility>
 #include <initializer_list>
@@ -54,7 +55,7 @@
 #include "../zuluCrypt-gui/utility.h"
 #include "siritask.h"
 #include "zulumounttask.h"
-#include "task.h"
+#include "task.hpp"
 #include "../zuluCrypt-gui/checkforupdates.h"
 #include "../zuluCrypt-gui/favorites.h"
 
@@ -67,10 +68,17 @@ static bool _encrypted_folder( const QString& e,bool f = true )
 
 zuluMount::zuluMount( QWidget * parent ) :
 	QWidget( parent ),
-	m_mountInfo( this,true,[ this ](){ QCoreApplication::quit() ; } ),
+	m_mountInfo( this,true,[](){ QCoreApplication::quit() ; } ),
 	m_events( this,m_mountInfo.stop() ),
-	m_checkForUpdates( this )
+	m_checkForUpdates( this ),
+	m_signalHandler( this )
 {
+	m_signalHandler.setAction( [ this ]( systemSignalHandler::signal s ){
+
+		Q_UNUSED( s ) ;
+
+		this->emergencyQuitApplication() ;
+	} ) ;
 }
 
 void zuluMount::helperStarted( bool start,const QString& volume )
@@ -138,6 +146,20 @@ void zuluMount::helperStarted( bool start,const QString& volume )
 	connect( this,SIGNAL( unlistVolume( QString ) ),
 		 this,SLOT( removeVolume( QString ) ) ) ;
 
+	m_ui->tableWidget->setContextMenuPolicy( Qt::CustomContextMenu ) ;
+
+	connect( m_ui->tableWidget,&QTableWidget::customContextMenuRequested,[ this ]( QPoint s ){
+
+		Q_UNUSED( s ) ;
+
+		auto item = m_ui->tableWidget->currentItem() ;
+
+		if( item ){
+
+			this->showContextMenu( item,true ) ;
+		}
+	} ) ;
+
 	this->setUpShortCuts() ;
 
 	this->setUpFont() ;
@@ -157,6 +179,28 @@ void zuluMount::helperStarted( bool start,const QString& volume )
 	m_autoMount = this->autoMount() ;
 	m_autoOpenFolderOnMount = this->autoOpenFolderOnMount() ;
 
+	trayMenu->addAction( [ this ](){
+
+		auto ac = new QAction( tr( "Do Not Minimize To Tray" ),this ) ;
+
+		m_actionPair.append( { ac,"Do Not Minimize To Tray" } ) ;
+
+		ac->setCheckable( true ) ;
+		ac->setChecked( utility::doNotMinimizeToTray() ) ;
+
+		connect( ac,&QAction::triggered,[ ac ](){
+
+			auto s = !utility::doNotMinimizeToTray() ;
+
+			ac->setChecked( s ) ;
+			utility::setDoNotMinimizeToTray( s ) ;
+		} ) ;
+
+		m_autoMountAction = ac ;
+
+		return ac ;
+	}() ) ;
+
 	trayMenu->addAction( [ this ](){
 
 		auto ac = new QAction( tr( "Automount Volumes" ),this ) ;
@@ -200,7 +244,7 @@ void zuluMount::helperStarted( bool start,const QString& volume )
 
 	m_favorite_menu = [ this,trayMenu ](){
 
-		auto m = trayMenu->addMenu( tr( "Favorites" ) ) ;
+		auto m = new QMenu( tr( "Favorites" ),this ) ;
 
 		m_menuPair.append( { m,"Favorites" } ) ;
 
@@ -212,6 +256,8 @@ void zuluMount::helperStarted( bool start,const QString& volume )
 		connect( m,SIGNAL( aboutToShow() ),
 			 this,SLOT( showFavorites() ) ) ;
 
+		m_ui->pbfavorites->setMenu( m ) ;
+
 		return m ;
 	}() ;
 
@@ -365,6 +411,13 @@ void zuluMount::helperStarted( bool start,const QString& volume )
 
 		this->raiseWindow() ;
 	}
+
+	auto debugWindowShortCut = new QShortcut( this ) ;
+	debugWindowShortCut->setKey(Qt::CTRL | Qt::SHIFT | Qt::Key_D ) ;
+	connect( debugWindowShortCut,&QShortcut::activated,[ this ](){
+
+		m_debugWindow.Show() ;
+	} ) ;
 }
 
 void zuluMount::showTrayIcon( bool e )
@@ -728,6 +781,7 @@ void zuluMount::Show()
 			       utility::cmdArgumentValue( l,"-d" ),
 			       [ this ]( const QString& e ){ utility::startHelperExecutable( this,
 											     e,
+											     "zuluMount",
 											     "helperStarted",
 											     "closeApplication" ) ; },
 			       [ this ]( int s ){ this->closeApplication( s ) ;	},
@@ -756,7 +810,7 @@ void zuluMount::cryfsVolumeProperties()
 
 	struct statfs vfs ;
 
-	if( Task::await< int >( [ & ](){ return statfs( mountPath.toLatin1().constData(),&vfs ) ; } ) ){
+	if( Task::await( [ & ](){ return statfs( mountPath.toLatin1().constData(),&vfs ) ; } ) ){
 
 		msg.ShowUIOK( tr( "ERROR" ),tr( "Failed To Read Volume Properties" ) ) ;
 
@@ -1006,7 +1060,11 @@ void zuluMount::showContextMenu( QTableWidgetItem * item,bool itemClicked )
 				m.addAction( tr( "Properties" ) )->setEnabled( false ) ;
 			}
 		}else{
-			connect( m.addAction( tr( "Properties" ) ),SIGNAL( triggered() ),
+			auto ac = m.addAction( tr( "Properties" ) ) ;
+
+			//ac->setEnabled( fs != "crypto_BitLocker" ) ;
+
+			connect( ac,SIGNAL( triggered() ),
 				 this,SLOT( volumeProperties() ) ) ;
 		}
 
@@ -1079,6 +1137,7 @@ void zuluMount::showContextMenu( QTableWidgetItem * item,bool itemClicked )
 	m.addAction( tr( "Close Menu" ) ) ;
 
 	if( itemClicked ){
+
 		m.exec( QCursor::pos() ) ;
 	}else{
 		auto p = this->pos() ;
@@ -1203,7 +1262,13 @@ void zuluMount::setUpFont()
 void zuluMount::closeEvent( QCloseEvent * e )
 {
 	e->ignore() ;
+
 	this->hide() ;
+
+	if( utility::doNotMinimizeToTray() ){
+
+		this->closeApplication() ;
+	}
 }
 
 void zuluMount::slotTrayClicked( QSystemTrayIcon::ActivationReason e )
@@ -1512,6 +1577,44 @@ void zuluMount::pbUmount_powerDown()
 	this->unmount( m_powerOff ) ;
 }
 
+void zuluMount::emergencyQuitApplication()
+{
+	this->hide() ;
+
+	auto table = m_ui->tableWidget ;
+
+	m_removeAllVolumes = true ;
+
+	auto paths       = tablewidget::columnEntries( table,0 ) ;
+	auto mountPoints = tablewidget::columnEntries( table,1 ) ;
+	auto fileSystems = tablewidget::columnEntries( table,2 ) ;
+
+	auto mountPath = utility::mountPath( QString() ) ;
+	auto homeMountPath = utility::homeMountPath( QString() ) ;
+
+	for( int i = 0 ; i < paths.size() ; i++ ){
+
+		const auto& a = paths.at( i ) ;
+		const auto& b = mountPoints.at( i ) ;
+		const auto& c = fileSystems.at( i ) ;
+
+		if( utility::startsWithAtLeastOne( b,mountPath,homeMountPath ) ){
+
+			if( _encrypted_folder( c ) ){
+
+				if( siritask::encryptedFolderUnMount( a,b,c ).await() ){
+
+					siritask::deleteMountFolder( b ) ;
+				}
+			}else{
+				zuluMountTask::unmountVolume( a,c ).await() ;
+			}
+		}
+	}
+
+	this->closeApplication() ;
+}
+
 void zuluMount::unMountAll()
 {
 	this->disableAll() ;
@@ -1677,6 +1780,7 @@ void zuluMount::disableAll()
 	m_ui->pbupdate->setEnabled( false ) ;
 	m_ui->tableWidget->setEnabled( false ) ;
 	m_ui->pbunlockcryptfs->setEnabled( false ) ;
+	m_ui->pbfavorites->setEnabled( false ) ;
 }
 
 void zuluMount::enableAll()
@@ -1691,6 +1795,7 @@ void zuluMount::enableAll()
 	m_ui->pbmount->setEnabled( true ) ;
 	m_ui->tableWidget->setFocus() ;
 	m_ui->pbunlockcryptfs->setEnabled( true ) ;
+	m_ui->pbfavorites->setEnabled( true ) ;
 }
 
 void zuluMount::enableAll_1()
